@@ -8,6 +8,7 @@
 use 5.006;
 use strict;
 use warnings;
+use Getopt::Long;
 use JSON;
 use Net::Shadowsocks::Server;
 
@@ -16,17 +17,12 @@ open(my $fh, '<', 'config.json') or die "Can't open  config.json: $!";
 my $confg_json = <$fh>;
 close($fh) || warn "close failed: $!";
 my $config    = decode_json($confg_json);
-my $_tcp_host = $config->{'local_address'};
-
-my $_tcp_service   = $config->{'local_port'};
 my $_password      = $config->{'password'};
 my $_remotehost    = $config->{'server'};
 my $_remoteservice = $config->{'server_port'};
 my $_method = $config->{'method'};
 
 my $foo = Net::Shadowsocks::Server->new(
-                                local_address => $_tcp_host,
-                                local_port    => $_tcp_service,
                                 password      => $_password,
                                 server        => $_remotehost,
                                 server_port   => $_remoteservice,
