@@ -1,6 +1,6 @@
 package Net::Shadowsocks;
 
-# ABSTRACT: An asynchronous, non-blocking shadowsocks client and server module.
+# ABSTRACT: If unsure, choose rc4-md5. 
 use 5.006;
 use strict;
 use warnings;
@@ -12,15 +12,16 @@ use Crypt::Cipher::RC6;
 use Crypt::Mode::CBC;
 use Crypt::Mode::CFB;
 use Crypt::Mode::CTR;
+use Crypt::Mode::OFB;
 use Crypt::NaCl::Sodium;
-#use Crypt::Nettle::Cipher;
 use Crypt::Random qw( makerandom_octet );
 use Digest::MD5;
+use Digest::SHA;
 use Mcrypt qw(ARCFOUR RIJNDAEL_128 RIJNDAEL_192 RIJNDAEL_256);
 use Mcrypt qw(:MODES);
 use Mcrypt qw(:FUNCS);
 
-our $VERSION = '0.7.2';
+our $VERSION = '0.8.1';
 
 our %_ciphers = 
     (
@@ -37,12 +38,12 @@ our %_ciphers =
         "aes-256-cbc"  => [Mcrypt::RIJNDAEL_256,Mcrypt::CBC,32,16],
         "aes-256-cfb"  => ['AES','cfb',32,16],
         "aes-256-ctr"  => ['AES','ctr',32,16],
-        "aes-256-ofb"  => [Mcrypt::RIJNDAEL_256,Mcrypt::OFB,32,16]
-        ,"camellia-128-cbc" => ['Camellia','cbc',16,16],
+        "aes-256-ofb"  => [Mcrypt::RIJNDAEL_256,Mcrypt::OFB,32,16],
+        "camellia-128-cbc" => ['Camellia','cbc',16,16],
         "camellia-128-cfb" => ['Camellia','cfb',16,16],
-         "camellia-128-ctr" => ['Camellia','ctr',16,16],
-          "camellia-128-ofb" => ['Camellia','ofb',16,16],
-          "camellia-192-cbc" => ['Camellia','cbc',24,16],
+        "camellia-128-ctr" => ['Camellia','ctr',16,16],
+        "camellia-128-ofb" => ['Camellia','ofb',16,16],
+        "camellia-192-cbc" => ['Camellia','cbc',24,16],
         "camellia-192-cfb" => ['Camellia','cfb',24,16],
         "camellia-192-ctr" => ['Camellia','ctr',24,16],
         "camellia-192-ofb" => ['Camellia','ofb',24,16],
@@ -128,7 +129,7 @@ sub _initialize_cipher($$)
                 }
             elsif ( $_method eq 'rc6') 
             {
-                $_key = Net::Shadowsocks::_EVP_BytesToKey($_ciphers{$_method}->[2], $_ciphers{$_method}->[3] , '', $_password, 1 );
+                $_key = _EVP_BytesToKey($_ciphers{$_method}->[2], $_ciphers{$_method}->[3] , '', $_password, 1 );
                 $_iv = makerandom_octet( Length => $_ciphers{$_method}->[3] );
                 $_encryptor = Crypt::Mode::CBC->new('RC6');
                 $_decryptor = Crypt::Mode::CBC->new('RC6');
@@ -147,7 +148,32 @@ sub _initialize_cipher($$)
                 $_encryptor = Crypt::Mode::CTR->new($_ciphers{$_method}->[0]);
                 $_decryptor = Crypt::Mode::CTR->new($_ciphers{$_method}->[0]);
             }
-            elsif(defined($_method) and $_method !~ /cfb$/ and $_method !~ /ctr$/ )
+             elsif(defined($_method) and $_method =~ /^camellia/)
+            {
+                $_key = _EVP_BytesToKey( $_ciphers{$_method}->[2], $_ciphers{$_method}->[3], '', $_password, 1 );
+                $_iv = makerandom_octet( Length => $_ciphers{$_method}->[3] );
+                if ($_method =~ /cbc$/)
+                { 
+                $_encryptor = Crypt::Mode::CBC->new($_ciphers{$_method}->[0]);
+                $_decryptor = Crypt::Mode::CBC->new($_ciphers{$_method}->[0]);
+                }
+                elsif($_method =~ /cfb$/)
+                {
+                $_encryptor = Crypt::Mode::CFB->new($_ciphers{$_method}->[0]);
+                $_decryptor = Crypt::Mode::CFB->new($_ciphers{$_method}->[0]);
+                }
+                elsif($_method =~ /ctr$/)
+                {
+                $_encryptor = Crypt::Mode::CTR->new($_ciphers{$_method}->[0]);
+                $_decryptor = Crypt::Mode::CTR->new($_ciphers{$_method}->[0]);
+                }
+                elsif($_method =~ /ofb$/)
+                {
+                $_encryptor = Crypt::Mode::OFB->new($_ciphers{$_method}->[0]);
+                $_decryptor = Crypt::Mode::OFB->new($_ciphers{$_method}->[0]);
+                }
+            }
+            elsif(defined($_method) and $_method !~ /cfb$/ and $_method !~ /ctr$/  and $_method !~ /^camellia/)
             {
                 $_encryptor = Mcrypt::mcrypt_load($_ciphers{$_method}->[0],'',$_ciphers{$_method}->[1] , '' );
                 $_decryptor = Mcrypt::mcrypt_load($_ciphers{$_method}->[0] ,'',$_ciphers{$_method}->[1], '' );
@@ -202,11 +228,16 @@ Net::Shadowsocks
 
 =head1 VERSION
 
-Version 0.7.2
+Version 0.8.1
 
 =head1 SYNOPSIS
 
-    Shadowsocks protocol client and server module.
+    ssclient.pl is the asynchronous, non-blocking shadowsocks client. 
+    
+    
+    
+    ssserver.pl is the asynchronous, non-blocking shadowsocks server. 
+    Run ssclient.pl and/or ssserver.pl and follow instructions.
     
 =head1 DESCRIPTION
 
