@@ -17,7 +17,7 @@ package Net::Shadowsocks::Client;
     use Net::Shadowsocks qw( _get_algorithm _get_mode _EVP_BytesToKey _initialize_cipher);
     use Socket qw(IPPROTO_TCP MSG_FASTOPEN SOL_SOCKET SO_REUSEPORT SO_REUSEADDR);
 
-    our $VERSION = '0.8.0';
+    our $VERSION = '0.8.1';
     $AnyEvent::Log::FILTER->level ("info");
     sub new($$$$$$) 
     {
@@ -29,9 +29,12 @@ package Net::Shadowsocks::Client;
             map { ( $_ => $args{$_} ) }
               qw(local_address local_port password server server_port method),
         }, $class;
-        if ( $self->{local_address} eq 'localhost' ) 
+        if (defined($self->{local_address}))
         {
+             if( $self->{local_address} eq 'localhost' ) 
+            {
             undef $self->{local_address};
+            }
         }
 
         my $tcp_server;
@@ -49,8 +52,6 @@ package Net::Shadowsocks::Client;
                 my $stage = 0;
                 my $mode  = 0;
                 my ( $encryptor, $decryptor, $key, $iv, $nonce ) = Net::Shadowsocks::_initialize_cipher( $self->{method},$self->{password} );
-                carp $key;
-                carp $iv;
                 my $clienthandler;
                 my $remotehandler;
                 $clienthandler = AnyEvent::Handle->new
@@ -61,18 +62,19 @@ package Net::Shadowsocks::Client;
                     fh        => $client_socket,
                     on_eof    => sub 
                     {
-                        my $chandler = shift;
+                        my $chandle = shift;
                         AE::log info => "Client: Done.";
-                        $chandler->destroy();
+                        $chandle->destroy();
                     },
                     on_error => sub 
                     {
-                        my ( $chandler, $fatal, $msg ) = @_;
+                        my ( $chandle, $fatal, $msg ) = @_;
                         AE::log error => $msg;
-                        $chandler->destroy();
+                        $chandle->destroy();
                     },
                     on_read => sub 
                     {
+                        my $chandle = shift;
                         my $client_buffer = $clienthandler->rbuf();
 
                         if ( $client_buffer eq '' ) 
@@ -114,8 +116,9 @@ package Net::Shadowsocks::Client;
                                     my $addrtype = ord( substr( $client_buffer, 3, 1 ) );
                                     if (    $addrtype != 1 and $addrtype != 3 and $addrtype != 4 )
                                     {
-                                        carp 'addrtype not supported';
-                                        return;
+                                          AE::log error => "'Invaliad address type'";
+                                          $chandle->destroy();
+                                        #return;
                                     }
                                     $addr_to_send = substr( $client_buffer, 3 );
                                     if ( !defined( $self->{local_address} ) ) 
@@ -149,28 +152,28 @@ package Net::Shadowsocks::Client;
                                         on_eof => sub 
                                         {
                                             my $rhandle = shift;
+                                            #AE::log info => "Remote: Done.";
                                             $rhandle->destroy();
                                             $mode = 0;
-                                            AE::log info => "Remote: Done.";
                                         },
                                         on_error => sub 
                                         {
                                             my ( $rhandle, $fatal, $msg ) = @_;
                                             AE::log error => $msg;
-                                            $mode = 0;
                                             $rhandle->destroy();
+                                            $mode = 0;
                                         },
                                         on_prepare => sub
                                         {
                                             my $phandle = shift;
                                             setsockopt($phandle->{fh}, SOL_SOCKET, SO_REUSEPORT, 1);
-                                            },
+                                        },
                                         on_read => sub 
                                         {
                                             my $incomingdata = $remotehandler->rbuf();
                                             my $decrypteddata;
-                                                unless ( defined($server_iv) ) 
-                                                {
+                                            unless ( defined($server_iv) ) 
+                                             {
                                                     $server_iv = substr( $incomingdata, 0,length($iv) );
                                                     $incomingdata = substr($incomingdata,length($iv));
                                                     if ( !defined($self->{method}) or $self->{method} eq'rc4-md5' )
@@ -188,36 +191,36 @@ package Net::Shadowsocks::Client;
                                                         #$md->add($server_iv);
                                                         my $decrypt_rc4_key = substr($sha->digest(),0,16);
                                                         Mcrypt::mcrypt_init($decryptor,$decrypt_rc4_key,'' );
-                                                        }
+                                                    }
 =for comment
                                                             elsif($self->{method} =~ /ctr$/)
                                                             {
                                                                 $decryptor =   Crypt::Nettle::Cipher->new('decrypt', Net::Shadowsocks::_get_algorithm($self->{method}), $key, Net::Shadowsocks::_get_mode($self->{method}) ,$server_iv);                                                                
                                                                 }
 =cut
-                                                                  elsif($self->{method} ne'rc6' and $self->{method} ne 'chacha20-ietf' and $self->{method} != /cfb$/ and $self->{method} != /ctr$/  )
-                                                        {
+                                                    elsif($self->{method} ne'rc6' and $self->{method} ne 'chacha20-ietf' and $self->{method} !~ /cfb$/ and $self->{method} !~ /ctr$/  and $self->{method} !~ /^camellia/ )
+                                                    {
                                                             Mcrypt::mcrypt_init($decryptor,$key,$server_iv );
-                                                            }
-                                                }
-                                                if ($self->{method} eq 'rc6' or $self->{method} =~ /cfb$/ or $self->{method} =~ /ctr$/)
-                                                {
-                                                    $decrypteddata = $decryptor->decrypt($incomingdata,$key,$server_iv);
                                                     }
+                                              }
+                                              if ($self->{method} eq 'rc6' or $self->{method} =~ /cfb$/ or $self->{method} =~ /ctr$/ or $self->{method} =~ /^camellia/)
+                                              {
+                                                    $decrypteddata = $decryptor->decrypt($incomingdata,$key,$server_iv);
+                                              }
 
-                                                    elsif($self->{method} eq 'chacha20-ietf')
-                                                    {
+                                              elsif($self->{method} eq 'chacha20-ietf')
+                                              {
                                                         $decrypteddata = $decryptor->chacha20_ietf_xor($incomingdata,$server_iv,$key);
-                                                        }
+                                              }
 
-                                                    else
-                                                    {
-                                                $decrypteddata = Mcrypt::mcrypt_decrypt($decryptor, $incomingdata );
-                                            }
-                                            $clienthandler->push_write($decrypteddata);
-                                            $remotehandler->{rbuf} = '';
-                                        },
-                                    );
+                                              else
+                                              {
+                                                    $decrypteddata = Mcrypt::mcrypt_decrypt($decryptor, $incomingdata );
+                                              }
+                                              $clienthandler->push_write($decrypteddata);
+                                              $remotehandler->{rbuf} = '';
+                                         },
+                                     );
                                     $stage = 5;
                                 }
                                 elsif ( $cmd == CMD_BIND ) 
@@ -253,7 +256,7 @@ package Net::Shadowsocks::Client;
                                              $plaindata = $addr_to_send . $client_buffer;
                                             $addr_to_send = '';
                                      }
-                                      if ( $self->{method} eq 'rc6' or $self->{method} =~ /cfb$/ or $self->{method} =~ /ctr$/) 
+                                      if ( $self->{method} eq 'rc6' or $self->{method} =~ /cfb$/ or $self->{method} =~ /ctr$/ or $self->{method} =~ /^camellia/) 
                                     {
                                                                     
                                           $encrypteddata = $encryptor->encrypt($plaindata,$key,$iv);
@@ -314,7 +317,7 @@ Net::Shadowsocks::Client
 
 =head1 VERSION
 
-Version 0.8.0
+Version 0.8.1
 
 =head1 SYNOPSIS
 
