@@ -16,11 +16,11 @@
     use Net::Shadowsocks qw(_get_algorithm _get_mode _EVP_BytesToKey _initialize_cipher);
     use Socket qw(IPPROTO_TCP TCP_FASTOPEN SOL_SOCKET SO_REUSEPORT SO_REUSEADDR);
 
-    our $VERSION = '0.8.0';
+    our $VERSION = '0.8.1';
     
     $AnyEvent::Log::FILTER->level ("info");
     
-    sub new($$$$$$)
+    sub new($$$$)
      {
         my $_osname = $Config{osname};
         my $_osvers = $Config{osvers};
@@ -28,7 +28,7 @@
         my ( $class, %args ) = @_;
         my $self = bless {
             map { ( $_ => $args{$_} ) }
-              qw(local_address local_port password server server_port method),
+              qw(password server server_port method),
         }, $class;
         if ( $self->{server} eq 'localhost' )
          {
@@ -101,20 +101,19 @@
                                         my $decrypt_rc4_key = substr($sha->digest(),0,16);
                                         Mcrypt::mcrypt_init($decryptor, $decrypt_rc4_key, '' );
                                         }
-                                        elsif($self->{method} ne 'rc6' and $self->{method} ne 'chacha20-ietf' and  $self->{method} !~ /cfb$/ and $self-> {method} !~ /ctr$/ )
+                                        elsif($self->{method} ne 'rc6' and $self->{method} ne 'chacha20-ietf' and  $self->{method} !~ /cfb$/ and $self-> {method} !~ /ctr$/ and $self->{method} !~ /^camellia/ )
                                         {
                                             Mcrypt::mcrypt_init($decryptor, $key, $client_iv );
                                             }
                                             else
                                             {
                                                 }                                    
-                                    if ($self->{method} eq 'rc6' or $self->{method} =~ /cfb$/ or $self->{method} =~ /ctr$/)
+                                    if ($self->{method} eq 'rc6' or $self->{method} =~ /cfb$/ or $self->{method} =~ /ctr$/ or $self->{method} =~ /^camellia/)
                                     {
                                     $decrypteddata = $decryptor->decrypt($incomingdata,$key,$client_iv);
                                         }
                                            elsif($self->{method} eq 'chacha20-ietf')
                                        {
-                                           carp "chacha20 ietf mode";
                                            #$decrypteddata = $decryptor->process($incomingdata);
                                            $decrypteddata = $decryptor->chacha20_ietf_xor($incomingdata,$client_iv,$key);
                                        }
@@ -178,22 +177,25 @@
                                         last;
                                     }
                                 }
-                                 carp $dest_addr;
-                                 carp $dest_port;
 
                                 if ( $dest_addr eq '' ) 
                                 {
-                                    carp "dest_addr empty";
                                     last;
                                 }
-                                $remotehandler = AnyEvent::Handle->new
-                                (
-                                    autocork   => 1,
-                                    keepalive  => 1,
-                                    no_delay   => 1,
-                                    connect    => [ $dest_addr, $dest_port ],
-                                    on_connect => sub 
-                                    {
+=for comment
+                                AnyEvent::Socket::resolve_sockaddr($dest_addr, $dest_port, "tcp", 0, undef, sub 
+                                {
+                                    my ( $ip_addr ) = @_;
+                                    AE::log  info => "resolved " . AnyEvent::Socket::format_address( $ip_addr );
+=cut
+                                    $remotehandler = AnyEvent::Handle->new
+                                    (
+                                        autocork   => 1,
+                                        keepalive  => 1,
+                                        no_delay   => 1,
+                                        connect    => [ $dest_addr, $dest_port ],
+                                        on_connect => sub 
+                                        {
                                         my ($rhandle,  $peerhost,$peerport, $retry) = @_;
 
                                         unless ($rhandle) 
@@ -202,32 +204,32 @@
                                             return;
                                         }
                                     },
-                                    on_eof => sub 
-                                    {
+                                        on_eof => sub 
+                                        {
                                         my $rhandler = shift;
                                         AE::log info => " Remote : Done . ";
                                         $mode = 0;
                                         $rhandler->destroy();
                                     },
-                                    on_error => sub 
-                                    {
+                                        on_error => sub 
+                                        {
                                         my ( $rhandler, $fatal, $msg ) = @_;
                                         AE::log error => $msg;
                                         $mode = 0;
                                         $rhandler->destroy();
                                     },
-                                     on_prepare => sub
+                                        on_prepare => sub
                                         {
                                             my $phandle = shift;
                                             setsockopt($phandle->{fh}, SOL_SOCKET, SO_REUSEPORT, 1);
                                             },
-                                    on_read => sub 
-                                    {
+                                        on_read => sub 
+                                        {
                                         my $remote_buffer = $remotehandler->rbuf();
                                         my $plaindata = $remote_buffer;
                                         my $encrypteddata;
 
-                                            if ($self->{method} eq 'rc6' or $self->{method} =~ /cfb$/ or $self->{method} =~ /ctr$/)
+                                            if ($self->{method} eq 'rc6' or $self->{method} =~ /cfb$/ or $self->{method} =~ /ctr$/ or $self->{method} =~ /^camellia/)
                                             {
                                                 $encrypteddata = $encryptor->encrypt($plaindata,$key,$iv); 
                                                 }
@@ -255,11 +257,13 @@
                                         $clienthandler->push_write($datatosend);
                                         $remotehandler->{rbuf} = '';
                                     }
-                                );
+                                   );
+#                                }
+#                                );
                             }
                             else 
                             {
-                                if ( $self->{method} eq 'rc6'  or $self->{method} =~ /cfb$/ or $self->{method} =~ /ctr$/) 
+                                if ( $self->{method} eq 'rc6'  or $self->{method} =~ /cfb$/ or $self->{method} =~ /ctr$/ or $self->{method} =~ /^camellia/) 
                                 {
                                     $decrypteddata = $decryptor->decrypt($incomingdata,$key,$client_iv);
                                 }
@@ -282,17 +286,17 @@
             },
             sub 
             {
-                my $fh = shift;
-                setsockopt($fh, SOL_SOCKET, SO_REUSEADDR, 1);
+                my $client_socket = shift;
+                setsockopt($client_socket,,SOL_SOCKET, SO_REUSEADDR, 1);
                 if ( $_osname eq 'linux' ) 
                 {
                     my $_tfo = do 
                     {
                         local ( @ARGV, $/ ) = '/proc/sys/net/ipv4/tcp_fastopen';<>;
                     };
-                    if ( $_tfo eq '2' or $_tfo eq '3' ) 
+                    if ( $_tfo == 2 or $_tfo == 3 ) 
                     {
-                        setsockopt( $fh, IPPROTO_TCP, TCP_FASTOPEN, 1 );
+                        setsockopt( $client_socket, IPPROTO_TCP, TCP_FASTOPEN, 1 );
                         AE::log info => "TCP Fast Open enabled on server.";
                     }
                 }
@@ -301,7 +305,7 @@
                     my $_version_major = substr( $_osvers, 0, index( $_osvers, '.' ) );
                     if ( $_version_major >= 15 ) 
                     {
-                        setsockopt( $fh, IPPROTO_TCP, TCP_FASTOPEN, 1 );
+                        setsockopt( $client_socket, IPPROTO_TCP, TCP_FASTOPEN, 1 );
                         AE::log info => "TCP Fast Open enabled on server.";
                     }
                 }
@@ -325,7 +329,7 @@ Net::Shadowsocks::Server
 
 =head1 VERSION
 
-Version 0.8.0
+Version 0.8.1
 
 =head1 SYNOPSIS
 
@@ -373,8 +377,6 @@ Project website https://osdn.net/projects/ssperl/
     use Net::Shadowsocks::Server;
 
     my $foo = Net::Shadowsocks::Server->new(
-    local_address => ' localhost ',
-    local_port => 1491,
     password => ' 49923641 ',
     server => ' jp . ssip . club ',
     server_port => 23333,
